//	Darkstar Engine  (c) UnderWorld Interactive Studio (all coding by Svarog    2007-2008)
//
//          Darkstar Engine.    
//        ,       ( -
//          ).
//
//       : effect_common.cpp
//       : Effect. Common for all platforms functions
//

#include <Darkstar/Graphic System/Effect/effect.hpp>
#include <Darkstar/Kernel/logh.hpp>
#include <Darkstar/Graphic System/render_state.hpp>
#include <Darkstar/Graphic System/Effect/effect_manager.hpp>
#include <Darkstar/Graphic System/Direct3D/graphicDriver.hpp>

using namespace gs;
using ker::Log;

Technique::Technique()
{
	const_table.reset(new ConstantTable());
};

Effect::Effect()
{
	cur_ce_index = 0;
	need_to_get_ce_index = true;
	cur_technique.reset(new Technique());
};

Effect::~Effect()
{
	techniques.clear();
};

void Effect::addNewTechnique(const std::string& name, const shared_ptr<Technique>& technique)
{
	techniques.insert(std::pair<std::string, shared_ptr<Technique> >(name, technique));
};

bool Effect::setCurTechnique(const std::string& tname)
{
	hmt_it it = techniques.find(tname);
	if(it == techniques.end())
	{
		lwar << "Cannot find technique: " << tname;
		return false;
	};
	cur_technique = it->second;
	need_to_get_ce_index = true;
	return true;
};

bool Effect::getIsTechniqueExist(const std::string& tname) const
{
	const_hmt_it it = techniques.find(tname);
	return it != techniques.end();
};

void Effect::setMacro(const std::string& name, short new_value)
{
	stdext::hash_map<std::string, unsigned int>::iterator it = cur_technique->macros_num_map.find(name);
	if(it == cur_technique->macros_num_map.end())
	{
		lwar << "Cannot find macros: " << name;
		return;
	};
	Macro nm = cur_technique->macros[it->second];
	if(nm.max_value < new_value || nm.min_value > new_value)
	{	
		lwar << "Cannot set new value for macros: " << name << " value is out of range";
		return;
	};
	if(cur_technique->macros[it->second].cur_value == new_value)
		return;
	cur_technique->macros[it->second].cur_value = new_value;
	need_to_get_ce_index = true;
	return;
};

bool Effect::getIsMacroExist(const std::string& name) const
{
	stdext::hash_map<std::string, unsigned int>::iterator it = cur_technique->macros_num_map.find(name);
	return it != cur_technique->macros_num_map.end();
};

short Effect::getMacroCurValue(const std::string& name) const
{
	stdext::hash_map<std::string, unsigned int>::iterator it = cur_technique->macros_num_map.find(name);
	if(it == cur_technique->macros_num_map.end())
	{
		lmes << "Try to get current value of not exist macros " << name;
		return -100;
	};
	return cur_technique->macros[it->second].cur_value;
};

short Effect::getMacroMaxValue(const std::string& name) const
{
	stdext::hash_map<std::string, unsigned int>::iterator it = cur_technique->macros_num_map.find(name);
	if(it == cur_technique->macros_num_map.end())
	{
		lmes << "Try to get max value of not exist macros: " << name;
		return -100;
	};
	return cur_technique->macros[it->second].max_value;
};

short Effect::getMacroMinValue(const std::string& name) const
{
	stdext::hash_map<std::string, unsigned int>::iterator it = cur_technique->macros_num_map.find(name);
	if(it == cur_technique->macros_num_map.end())
	{
		lmes << "Try to get min value of not exist macros " << name;
		return -100;
	};
	return cur_technique->macros[it->second].min_value;
};

void Effect::_getCurCeIndex()
{
	unsigned int number = 0;
	std::vector<Macro>::const_iterator it;
	for(it = cur_technique->macros.begin(); it != cur_technique->macros.end(); ++it)
		number += it->contribution*(it->cur_value - it->min_value);
	cur_ce_index = number;
	need_to_get_ce_index = false;
};

bool Effect::setSamplers()
{
	stdext::hash_map<std::string, shared_ptr<SamplerRenderState> >::iterator it;
	for(it = cur_technique->samplers.begin(); it != cur_technique->samplers.end(); ++it)
	{
		if(!getIsConstantExist(it->first))
			continue;
		short ci = getConstantIndex(it->first);
		it->second->set(ci, (SAMPLER_TYPE)cur_technique->const_table->getConstantType(it->first));
	};
	return true;
};

bool Effect::getIsConstantExist(const std::string& name) const
{
	return cur_technique->const_table->getIsConstantExist(name);
};

short Effect::getConstantIndex(const std::string& name) const
{
	return cur_technique->const_table->getConstantIndex(name);
};

void Effect::set()
{
	if(need_to_get_ce_index)
		_getCurCeIndex();
#ifdef _NOTFINAL
	if(cur_ce_index >= cur_technique->ce.size() || cur_technique->ce.size() == 0)
	{
		lwar << "Cur Concrete effect index greater then concrete effects size";
		return;
	};
	if(!cur_technique->ce[cur_ce_index].ps || !cur_technique->ce[cur_ce_index].vs)
	{
		lwar << "Pixel or vertex shader in conrete effect is NULL";
		return;
	};
#endif
	cur_technique->ce[cur_ce_index].ps->set();
	cur_technique->ce[cur_ce_index].vs->set();
};

const InputSignature& Effect::getEffectInputSignature() const
{
	return input_signature;
};

void Effect::setEffectInputSignature(const InputSignature& input_signature) 
{
	this->input_signature = input_signature;
};


	

