//	Darkstar Engine  (c) UnderWorld Interactive Studio (all coding by Svarog    2007-2008)
//
//          Darkstar Engine.    
//        ,       ( -
//          ).
//
//       : point_light.hpp
//       : point light
//

#ifndef DSE_GS_LIGHT_SYSTEM_POINT_LIGHT_HPP
#define DSE_GS_LIGHT_SYSTEM_POINT_LIGHT_HPP

#include <Darkstar/common.hpp>
#include <Darkstar/Graphic System/Light System/light_common.hpp>
#include <Darkstar/Math/sphere.hpp>

namespace gs
{
	class PointLight
	{
	private:
		DRSLIGHTSTATE light_state;
		float attenuation;
		float range;
		math::Vect3 diffuse_color;
		math::Vect3 specular_color;
		math::Vect3 ambient_color;

		math::Vect3 position;
		math::Sphere volume;
	public:
		PointLight();
		PointLight(const DRSLIGHTSTATE& light_state, const math::Vect3& position, 
			const float range, const math::Vect3& diffuse_color = math::Vect3(1, 1, 1), 
			const math::Vect3& specular_color = math::Vect3(1, 1, 1), 
			const math::Vect3& ambient_color = math::Vect3(0, 0, 0), const float attenuation = 0);
		~PointLight();

		void setLightState(const DRSLIGHTSTATE& new_light_state);
		const DRSLIGHTSTATE& getLightState() const;
		void toggleLightState();

		void setDiffuseColor(const math::Vect3& new_color);
		const math::Vect3& getDiffuseColor() const;
		void setAmbientColor(const math::Vect3& new_color);
		const math::Vect3& getAmbientColor() const;
		void setSpecularColor(const math::Vect3& new_color);
		const math::Vect3& getSpecularColor() const;

		void setRange(const float new_range);
		const float getRange() const;
		void setAttenuation(const float attenuation);
		const float getAttenuation() const;

		const math::Sphere& getVolume() const;

		const math::Vect3& getPosition() const;
		void setPosition(const math::Vect3& new_position);

	};
};

#include <Darkstar/Graphic System/Light System/point_light.inl>

#endif