//	Darkstar Engine  (c) UnderWorld Interactive Studio (all coding by Svarog    2007-2008)
//
//          Darkstar Engine.    
//        ,       ( -
//          ).
//
//       : render_target_d3d10.hpp
//       : Render Target object for direct3d 10
//

#ifdef GD_D3D10

#ifndef DSE_GS_RT_RENDER_TARGET_D3D10_HPP
#define DSE_GS_RT_RENDER_TARGET_D3D10_HPP

#include <Darkstar/common.hpp>
#include <Darkstar/Graphic System/Render Target/render_target_common.hpp>
#include <Darkstar/Resource System/Textures/rtTexture.hpp>

namespace gs
{
	class RenderTarget
	{
	private:
		unsigned int layers_number;

		std::vector<ID3D10RenderTargetView*> render_target_views;
		std::vector<ID3D10DepthStencilView*> depth_stencil_view;
		ID3D10Texture2D* texture_data;
		ID3D10Texture2D* texture_data_without_aa;
		shared_ptr<rs::RTTexture> darkstar_texture;

		unsigned int width;
		unsigned int height;
		DRTAATYPE aa_type;
		bool need_texture;
		DRTFORMAT format;
	public:
		RenderTarget();
		~RenderTarget();

		bool create2D(const unsigned int width, const unsigned int height, const DRTFORMAT& format, 
			const DRTAATYPE& aa_type, bool need_texture);
		bool create2D(ID3D10Texture2D* rt_tex);

		bool createCube(const unsigned int width, const unsigned int height, const DRTFORMAT& format);

		unsigned int getWidth() const;
		unsigned int getHeight() const;
		const DRTAATYPE& getAAType() const;
		const DRTFORMAT& getFormat() const;
		bool getIsNeedTexture() const;

		unsigned int getLayersNumber() const;

		shared_ptr<rs::BaseTexture> getTexture() const;

		ID3D10RenderTargetView* getRenderTargetView(const unsigned int layer_number = 0) const;
		ID3D10DepthStencilView* getDepthStencilView(const unsigned int layer_number = 0) const;
		ID3D10Texture2D* getDataTexture() const;
		ID3D10Texture2D* getDataWithoutAATexture() const;
	};
};

#endif

#endif