//	Darkstar Engine  (c) UnderWorld Interactive Studio (all coding by Svarog    2007-2008)
//
//          Darkstar Engine.    
//        ,       ( -
//          ).
//
//       : vertex_stream_d3d9.cpp
//       :    Direct3D 9
//

#ifdef GD_D3D9

#include <Darkstar/Resource System/Model/vertex_stream_d3d9.hpp>
#include <Darkstar/Kernel/logh.hpp>
#include <Darkstar/Graphic System/Direct3D/graphicDriver.hpp>

using namespace rs;
using gs::GraphicDriver;
using ker::Log;

VertexStream::VertexStream()
{
	stream_size = 0;
	stream_element_size = 0;
	elems_count = 0;
	stream = NULL;
};

VertexStream::VertexStream(const unsigned int stream_elem_size, const unsigned int elems_count, const DEUSAGE& usage)
{
	stream = NULL;
	create(stream_elem_size, elems_count, usage);
};

VertexStream::~VertexStream()
{
	if(stream)
	{
		if(FAILED(stream->Release()))
			lwar << "Cannot release vertex stream";
		stream = NULL;
	};
};

bool VertexStream::create(const unsigned int stream_elem_size, const unsigned int elems_count, const DEUSAGE& usage /* = DEU_DYNAMIC */)
{
	if(stream)
	{
		lwar << "Vertex stream have already created";
		return false;
	};
	stream_size = stream_elem_size*elems_count;
	if(FAILED(GraphicDriver::ptr()->getDevice()->CreateVertexBuffer(stream_size, D3DUSAGE_DYNAMIC | D3DUSAGE_WRITEONLY, 0, 
		D3DPOOL_DEFAULT, &stream, NULL)))
	{
		lwar << "Cannot create vertex buffer";
		return false;
	};
	stream_element_size = stream_elem_size;
	this->elems_count = elems_count;
	is_dynamic = true;
	return true;
};

bool VertexStream::createWithData(const unsigned int stream_elem_size, const unsigned int elems_count, 
								  void* data, const DEUSAGE& usage)
{
	if(stream)
	{
		lwar << "Vertex stream have already created";
		return false;
	};
	stream_size = stream_elem_size*elems_count;
	if(FAILED(GraphicDriver::ptr()->getDevice()->CreateVertexBuffer(stream_size, 0, 0, 
		D3DPOOL_MANAGED, &stream, NULL)))
	{
		lwar << "Cannot create vertex buffer";
		return false;
	};
	stream_element_size = stream_elem_size;
	this->elems_count = elems_count;
	is_dynamic = false;
	upload(0, data, 0);
	return true;
};

bool VertexStream::upload(const unsigned int size, void* data, const unsigned int offset)
{
	if(!stream)
	{
		lwar << "Cannot upload uninitialized vertex stream";
		return false;
	};
	void* ptr;
	unsigned int up_size = size;
	if(size == 0)
		up_size = elems_count*stream_element_size;
	if(!(ptr = map(DEMT_WRITE_DISCARD)))
	{
		lwar << "Cannot lock vertex stream";
		return false;
	};
	memcpy((char*)ptr + offset, (char*)data, up_size);
	unmap();
	return true;
};

void* VertexStream::map(const DEMAPTYPE& map_type)
{
	DWORD flags = is_dynamic ? D3DLOCK_DISCARD : 0;
	void* ptr = NULL;;
	if(FAILED(stream->Lock(0, 0, &ptr, flags)))
	{
		lwar << "Cannot lock vertex stream";
		return NULL;
	};
	return ptr;
};

void VertexStream::unmap()
{
	stream->Unlock();
}

IDirect3DVertexBuffer9* VertexStream::getStream() const
{
	return stream;
};

unsigned int VertexStream::getElementSize() const
{
	return stream_element_size;
};

unsigned int VertexStream::getElementsCount() const
{
	return elems_count;
};

#endif